@@ -1,9 +1,12 @@
 """Events API module."""
 import logging
+import time
 
 from splitio.api import APIException
 from splitio.api.client import HttpClientException
-from splitio.api.commons import headers_from_metadata
+from splitio.api.commons import headers_from_metadata, record_telemetry
+from splitio.util.time import get_current_epoch_time_ms
+from splitio.models.telemetry import HTTPExceptionsAndLatencies
 
 
 _LOGGER = logging.getLogger(__name__)
@@ -12,7 +15,7 @@ _LOGGER = logging.getLogger(__name__)
 class EventsAPI(object):  # pylint: disable=too-few-public-methods
     """Class that uses an httpClient to communicate with the events API."""
 
-    def __init__(self, http_client, apikey, sdk_metadata):
+    def __init__(self, http_client, apikey, sdk_metadata, telemetry_runtime_producer):
         """
         Class constructor.
 
@@ -26,6 +29,7 @@ class EventsAPI(object):  # pylint: disable=too-few-public-methods
         self._client = http_client
         self._apikey = apikey
         self._metadata = headers_from_metadata(sdk_metadata)
+        self._telemetry_runtime_producer = telemetry_runtime_producer
 
     @staticmethod
     def _build_bulk(events):
@@ -61,14 +65,16 @@ class EventsAPI(object):  # pylint: disable=too-few-public-methods
         :rtype: bool
         """
         bulk = self._build_bulk(events)
+        start = get_current_epoch_time_ms()
         try:
             response = self._client.post(
                 'events',
                 '/events/bulk',
                 self._apikey,
                 body=bulk,
-                extra_headers=self._metadata
+                extra_headers=self._metadata,
             )
+            record_telemetry(response.status_code, get_current_epoch_time_ms() - start, HTTPExceptionsAndLatencies.EVENT, self._telemetry_runtime_producer)
             if not 200 <= response.status_code < 300:
                 raise APIException(response.body, response.status_code)
         except HttpClientException as exc:
