@@ -1,17 +1,19 @@
 """Redis storage module."""
 import json
 import logging
+import threading
 
 from splitio.models.impressions import Impression
 from splitio.models import splits, segments
+from splitio.models.telemetry import MethodExceptions, MethodLatencies, TelemetryConfig
 from splitio.storage import SplitStorage, SegmentStorage, ImpressionStorage, EventStorage, \
-    ImpressionPipelinedStorage
+    ImpressionPipelinedStorage, TelemetryStorage
 from splitio.storage.adapters.redis import RedisAdapterException
 from splitio.storage.adapters.cache_trait import decorate as add_cache, DEFAULT_MAX_AGE
 
 
 _LOGGER = logging.getLogger(__name__)
-
+MAX_TAGS = 10
 
 class RedisSplitStorage(SplitStorage):
     """Redis-based storage for splits."""
@@ -180,10 +182,17 @@ class RedisSplitStorage(SplitStorage):
             _LOGGER.debug('Error: ', exc_info=True)
             return []
 
+    def get_splits_count(self):
+        """
+        Return splits count.
+
+        :rtype: int
+        """
+        return 0
+
     def get_all_splits(self):
         """
         Return all the splits in cache.
-
         :return: List of all splits in cache.
         :rtype: list(splitio.models.splits.Split)
         """
@@ -345,6 +354,22 @@ class RedisSegmentStorage(SegmentStorage):
             _LOGGER.debug('Error: ', exc_info=True)
             return None
 
+    def get_segments_count(self):
+        """
+        Return segment count.
+
+        :return: 0
+        :rtype: int
+        """
+        return 0
+
+    def get_segments_keys_count(self):
+        """
+        Return segment count.
+
+        :rtype: int
+        """
+        return 0
 
 class RedisImpressionsStorage(ImpressionStorage, ImpressionPipelinedStorage):
     """Redis based event storage class."""
@@ -406,7 +431,6 @@ class RedisImpressionsStorage(ImpressionStorage, ImpressionPipelinedStorage):
         :type inserted: int
         """
         if total_keys == inserted:
-            _LOGGER.debug("SET EXPIRE KEY FOR QUEUE")
             self._redis.expire(self.IMPRESSIONS_QUEUE_KEY, self.IMPRESSIONS_KEY_DEFAULT_TTL)
 
     def add_impressions_to_pipe(self, impressions, pipe):
@@ -460,7 +484,8 @@ class RedisImpressionsStorage(ImpressionStorage, ImpressionPipelinedStorage):
 class RedisEventsStorage(EventStorage):
     """Redis based event storage class."""
 
-    _KEY_TEMPLATE = 'SPLITIO.events'
+    _EVENTS_KEY_TEMPLATE = 'SPLITIO.events'
+    _EVENTS_KEY_DEFAULT_TTL = 3600
 
     def __init__(self, redis_client, sdk_metadata):
         """
@@ -474,6 +499,38 @@ class RedisEventsStorage(EventStorage):
         self._redis = redis_client
         self._sdk_metadata = sdk_metadata
 
+    def add_events_to_pipe(self, events, pipe):
+        """
+        Add put operation to pipeline
+
+        :param impressions: List of one or more impressions to store.
+        :type impressions: list
+        :param pipe: Redis pipe.
+        :type pipe: redis.pipe
+        """
+        bulk_events = self._wrap_events(events)
+        pipe.rpush(self._EVENTS_KEY_TEMPLATE, *bulk_events)
+
+    def _wrap_events(self, events):
+        return [
+        json.dumps({
+            'e': {
+                'key': e.event.key,
+                'trafficTypeName': e.event.traffic_type_name,
+                'eventTypeId': e.event.event_type_id,
+                'value': e.event.value,
+                'timestamp': e.event.timestamp,
+                'properties': e.event.properties,
+            },
+            'm': {
+                's': self._sdk_metadata.sdk_version,
+                'n': self._sdk_metadata.instance_name,
+                'i': self._sdk_metadata.instance_ip,
+            }
+        })
+        for e in events
+    ]
+
     def put(self, events):
         """
         Add an event to the redis storage.
@@ -484,25 +541,8 @@ class RedisEventsStorage(EventStorage):
         :return: Whether the event has been added or not.
         :rtype: bool
         """
-        key = self._KEY_TEMPLATE
-        to_store = [
-            json.dumps({
-                'e': {
-                    'key': e.event.key,
-                    'trafficTypeName': e.event.traffic_type_name,
-                    'eventTypeId': e.event.event_type_id,
-                    'value': e.event.value,
-                    'timestamp': e.event.timestamp,
-                    'properties': e.event.properties,
-                },
-                'm': {
-                    's': self._sdk_metadata.sdk_version,
-                    'n': self._sdk_metadata.instance_name,
-                    'i': self._sdk_metadata.instance_ip,
-                }
-            })
-            for e in events
-        ]
+        key = self._EVENTS_KEY_TEMPLATE
+        to_store = self._wrap_events(events)
         try:
             self._redis.rpush(key, *to_store)
             return True
@@ -525,3 +565,160 @@ class RedisEventsStorage(EventStorage):
         Clear data.
         """
         raise NotImplementedError('Not supported for redis.')
+
+    def expire_keys(self, total_keys, inserted):
+        """
+        Set expire
+
+        :param total_keys: length of keys.
+        :type total_keys: int
+        :param inserted: added keys.
+        :type inserted: int
+        """
+        if total_keys == inserted:
+            self._redis.expire(self._EVENTS_KEY_TEMPLATE, self._EVENTS_KEY_DEFAULT_TTL)
+
+class RedisTelemetryStorage(TelemetryStorage):
+    """Redis based telemetry storage class."""
+
+    _TELEMETRY_CONFIG_KEY = 'SPLITIO.telemetry.init'
+    _TELEMETRY_LATENCIES_KEY = 'SPLITIO.telemetry.latencies'
+    _TELEMETRY_EXCEPTIONS_KEY = 'SPLITIO.telemetry.exceptions'
+    _TELEMETRY_KEY_DEFAULT_TTL = 3600
+
+    def __init__(self, redis_client, sdk_metadata):
+        """
+        Class constructor.
+
+        :param redis_client: Redis client or compliant interface.
+        :type redis_client: splitio.storage.adapters.redis.RedisAdapter
+        :param sdk_metadata: SDK & Machine information.
+        :type sdk_metadata: splitio.client.util.SdkMetadata
+        """
+        self._lock = threading.RLock()
+        self._reset_config_tags()
+        self._redis_client = redis_client
+        self._sdk_metadata = sdk_metadata
+        self._method_latencies = MethodLatencies()
+        self._method_exceptions = MethodExceptions()
+        self._tel_config = TelemetryConfig()
+        self._make_pipe = redis_client.pipeline
+
+    def _reset_config_tags(self):
+        with self._lock:
+            self._config_tags = []
+
+    def add_config_tag(self, tag):
+        """Record tag string."""
+        with self._lock:
+            if len(self._config_tags) < MAX_TAGS:
+                self._config_tags.append(tag)
+
+    def record_config(self, config, extra_config):
+        """
+        initilize telemetry objects
+
+        :param congif: factory configuration parameters
+        :type config: splitio.client.config
+        """
+        self._tel_config.record_config(config, extra_config)
+
+    def pop_config_tags(self):
+        """Get and reset tags."""
+        with self._lock:
+            tags = self._config_tags
+            self._reset_config_tags()
+            return tags
+
+    def push_config_stats(self):
+        """push config stats to redis."""
+        self._redis_client.hset(self._TELEMETRY_CONFIG_KEY, self._sdk_metadata.sdk_version + '/' + self._sdk_metadata.instance_name + '/' + self._sdk_metadata.instance_ip, str(self._format_config_stats()))
+
+    def _format_config_stats(self):
+        """format only selected config stats to json"""
+        config_stats = self._tel_config.get_stats()
+        return json.dumps({
+            'aF': config_stats['aF'],
+            'rF': config_stats['rF'],
+            'sT': config_stats['sT'],
+            'oM': config_stats['oM'],
+            't': self.pop_config_tags()
+        })
+
+    def record_active_and_redundant_factories(self, active_factory_count, redundant_factory_count):
+        """Record active and redundant factories."""
+        self._tel_config.record_active_and_redundant_factories(active_factory_count, redundant_factory_count)
+
+    def add_latency_to_pipe(self, method, latency, pipe):
+        """
+        record latency data
+
+        :param method: method name
+        :type method: string
+        :param latency: latency
+        :type latency: int64
+        :param pipe: Redis pipe.
+        :type pipe: redis.pipe
+        """
+        self._method_latencies.add_latency(method, latency)
+        latencies = self._method_latencies.pop_all()['methodLatencies']
+        values = latencies[method.value]
+        total_keys = 0
+        bucket_number = 0
+        for bucket in values:
+            if bucket > 0:
+                pipe.hincrby(self._TELEMETRY_LATENCIES_KEY, self._sdk_metadata.sdk_version + '/' + self._sdk_metadata.instance_name + '/' + self._sdk_metadata.instance_ip + '/' +
+                        method.value + '/' + str(bucket_number), bucket)
+                total_keys += 1
+            bucket_number = bucket_number + 0
+
+    def record_latency(self, method, latency):
+        """
+        Not implemented
+        """
+        raise NotImplementedError('Only redis pipe is used.')
+
+    def record_exception(self, method):
+        """
+        record an exception
+
+        :param method: method name
+        :type method: string
+        """
+        pipe = self._make_pipe()
+        pipe.hincrby(self._TELEMETRY_EXCEPTIONS_KEY, self._sdk_metadata.sdk_version + '/' + self._sdk_metadata.instance_name + '/' + self._sdk_metadata.instance_ip + '/' +
+                    method.value, 1)
+        result = pipe.execute()
+        self.expire_keys(self._TELEMETRY_EXCEPTIONS_KEY, self._TELEMETRY_KEY_DEFAULT_TTL, 1, result[0])
+
+    def record_not_ready_usage(self):
+        """
+        record not ready time
+
+        """
+        pass
+
+    def record_bur_time_out(self):
+        """
+        record BUR timeouts
+
+        """
+        pass
+
+    def record_impression_stats(self, data_type, count):
+        pass
+
+    def expire_latency_keys(self, total_keys, inserted):
+        self.expire_keys(self._TELEMETRY_LATENCIES_KEY, self._TELEMETRY_KEY_DEFAULT_TTL, total_keys, inserted)
+
+    def expire_keys(self, queue_key, key_default_ttl, total_keys, inserted):
+        """
+        Set expire
+
+        :param total_keys: length of keys.
+        :type total_keys: int
+        :param inserted: added keys.
+        :type inserted: int
+        """
+        if total_keys == inserted:
+            self._redis_client.expire(queue_key, key_default_ttl)
