@@ -3,6 +3,7 @@
 import json
 import os
 import threading
+import time
 import pytest
 
 from redis import StrictRedis
@@ -10,16 +11,20 @@ from redis import StrictRedis
 from splitio.client.factory import get_factory, SplitFactory
 from splitio.client.util import SdkMetadata
 from splitio.storage.inmemmory import InMemoryEventStorage, InMemoryImpressionStorage, \
-    InMemorySegmentStorage, InMemorySplitStorage
+    InMemorySegmentStorage, InMemorySplitStorage, InMemoryTelemetryStorage
 from splitio.storage.redis import RedisEventsStorage, RedisImpressionsStorage, \
-    RedisSplitStorage, RedisSegmentStorage
+    RedisSplitStorage, RedisSegmentStorage, RedisTelemetryStorage
 from splitio.storage.adapters.redis import build, RedisAdapter
 from splitio.models import splits, segments
 from splitio.engine.impressions.impressions import Manager as ImpressionsManager, ImpressionsMode
 from splitio.engine.impressions.strategies import StrategyDebugMode, StrategyOptimizedMode
 from splitio.engine.impressions.manager import Counter
+from splitio.engine.telemetry import TelemetryStorageConsumer, TelemetryStorageProducer
+from splitio.engine.impressions.manager import Counter as ImpressionsCounter
 from splitio.recorder.recorder import StandardRecorder, PipelinedRecorder
 from splitio.client.config import DEFAULT_CONFIG
+from splitio.sync.synchronizer import SplitTasks, SplitSynchronizers, Synchronizer
+from splitio.sync.manager import Manager
 
 class InMemoryIntegrationTests(object):
     """Inmemory storage-based integration tests."""
@@ -45,15 +50,32 @@ class InMemoryIntegrationTests(object):
             data = json.loads(flo.read())
         segment_storage.put(segments.from_raw(data))
 
+        telemetry_storage = InMemoryTelemetryStorage()
+        telemetry_producer = TelemetryStorageProducer(telemetry_storage)
+        telemetry_consumer = TelemetryStorageConsumer(telemetry_storage)
+        telemetry_runtime_producer = telemetry_producer.get_telemetry_runtime_producer()
+        telemetry_evaluation_producer = telemetry_producer.get_telemetry_evaluation_producer()
+
         storages = {
             'splits': split_storage,
             'segments': segment_storage,
-            'impressions': InMemoryImpressionStorage(5000),
-            'events': InMemoryEventStorage(5000),
+            'impressions': InMemoryImpressionStorage(5000, telemetry_runtime_producer),
+            'events': InMemoryEventStorage(5000, telemetry_runtime_producer),
         }
-        impmanager = ImpressionsManager(None, StrategyDebugMode()) # no listener
-        recorder = StandardRecorder(impmanager, storages['events'], storages['impressions'])
-        self.factory = SplitFactory('some_api_key', storages, True, recorder)  # pylint:disable=attribute-defined-outside-init
+        impmanager = ImpressionsManager(StrategyDebugMode(), telemetry_runtime_producer) # no listener
+        recorder = StandardRecorder(impmanager, storages['events'], storages['impressions'], telemetry_evaluation_producer)
+        # Since we are passing None as SDK_Ready event, the factory will use the Redis telemetry call, using try catch to ignore the exception.
+        try:
+            self.factory = SplitFactory('some_api_key',
+                                    storages,
+                                    True,
+                                    recorder,
+                                    None,
+                                    telemetry_producer=telemetry_producer,
+                                    telemetry_init_producer=telemetry_producer.get_telemetry_init_producer(),
+                                    )  # pylint:disable=attribute-defined-outside-init
+        except:
+            pass
 
     def teardown_method(self):
         """Shut down the factory."""
@@ -68,9 +90,19 @@ class InMemoryIntegrationTests(object):
         as_tup_set = set((i.feature_name, i.matching_key, i.treatment) for i in impressions)
         assert as_tup_set == set(to_validate)
 
+    def _validate_last_events(self, client, *to_validate):
+        """Validate the last N impressions are present disregarding the order."""
+        event_storage = client._factory._get_storage('events')
+        events = event_storage.pop_many(len(to_validate))
+        as_tup_set = set((i.key, i.traffic_type_name, i.event_type_id, i.value, str(i.properties)) for i in events)
+        assert as_tup_set == set(to_validate)
+
     def test_get_treatment(self):
         """Test client.get_treatment()."""
-        client = self.factory.client()
+        try:
+            client = self.factory.client()
+        except:
+            pass
 
         assert client.get_treatment('user1', 'sample_feature') == 'on'
         self._validate_last_impressions(client, ('sample_feature', 'user1', 'on'))
@@ -113,8 +145,10 @@ class InMemoryIntegrationTests(object):
 
     def test_get_treatment_with_config(self):
         """Test client.get_treatment_with_config()."""
-        client = self.factory.client()
-
+        try:
+            client = self.factory.client()
+        except:
+            pass
         result = client.get_treatment_with_config('user1', 'sample_feature')
         assert result == ('on', '{"size":15,"test":20}')
         self._validate_last_impressions(client, ('sample_feature', 'user1', 'on'))
@@ -139,8 +173,10 @@ class InMemoryIntegrationTests(object):
 
     def test_get_treatments(self):
         """Test client.get_treatments()."""
-        client = self.factory.client()
-
+        try:
+            client = self.factory.client()
+        except:
+            pass
         result = client.get_treatments('user1', ['sample_feature'])
         assert len(result) == 1
         assert result['sample_feature'] == 'on'
@@ -189,7 +225,10 @@ class InMemoryIntegrationTests(object):
 
     def test_get_treatments_with_config(self):
         """Test client.get_treatments_with_config()."""
-        client = self.factory.client()
+        try:
+            client = self.factory.client()
+        except:
+            pass
 
         result = client.get_treatments_with_config('user1', ['sample_feature'])
         assert len(result) == 1
@@ -237,9 +276,27 @@ class InMemoryIntegrationTests(object):
             ('sample_feature', 'invalidKey', 'off'),
         )
 
+    def test_track(self):
+        """Test client.track()."""
+        try:
+            client = self.factory.client()
+        except:
+            pass
+        assert(client.track('user1', 'user', 'conversion', 1, {"prop1": "value1"}))
+        assert(not client.track(None, 'user', 'conversion'))
+        assert(not client.track('user1', None, 'conversion'))
+        assert(not client.track('user1', 'user', None))
+        self._validate_last_events(
+            client,
+            ('user1', 'user', 'conversion', 1, "{'prop1': 'value1'}")
+        )
+
     def test_manager_methods(self):
         """Test manager.split/splits."""
-        manager = self.factory.manager()
+        try:
+            manager = self.factory.manager()
+        except:
+            pass
         result = manager.split('all_feature')
         assert result.name == 'all_feature'
         assert result.traffic_type is None
@@ -293,15 +350,28 @@ class InMemoryOptimizedIntegrationTests(object):
             data = json.loads(flo.read())
         segment_storage.put(segments.from_raw(data))
 
+        telemetry_storage = InMemoryTelemetryStorage()
+        telemetry_producer = TelemetryStorageProducer(telemetry_storage)
+        telemetry_consumer = TelemetryStorageConsumer(telemetry_storage)
+        telemetry_runtime_producer = telemetry_producer.get_telemetry_runtime_producer()
+        telemetry_evaluation_producer = telemetry_producer.get_telemetry_evaluation_producer()
+
         storages = {
             'splits': split_storage,
             'segments': segment_storage,
-            'impressions': InMemoryImpressionStorage(5000),
-            'events': InMemoryEventStorage(5000),
+            'impressions': InMemoryImpressionStorage(5000, telemetry_runtime_producer),
+            'events': InMemoryEventStorage(5000, telemetry_runtime_producer),
         }
-        impmanager = ImpressionsManager(None, StrategyOptimizedMode(Counter()))
-        recorder = StandardRecorder(impmanager, storages['events'], storages['impressions'])
-        self.factory = SplitFactory('some_api_key', storages, True, recorder)  # pylint:disable=attribute-defined-outside-init
+        impmanager = ImpressionsManager(StrategyOptimizedMode(ImpressionsCounter()), telemetry_runtime_producer) # no listener
+        recorder = StandardRecorder(impmanager, storages['events'], storages['impressions'], telemetry_evaluation_producer)
+        self.factory = SplitFactory('some_api_key',
+                                    storages,
+                                    True,
+                                    recorder,
+                                    None,
+                                    telemetry_producer=telemetry_producer,
+                                    telemetry_init_producer=telemetry_producer.get_telemetry_init_producer(),
+                                    )  # pylint:disable=attribute-defined-outside-init
 
     def _validate_last_impressions(self, client, *to_validate):
         """Validate the last N impressions are present disregarding the order."""
@@ -310,6 +380,13 @@ class InMemoryOptimizedIntegrationTests(object):
         as_tup_set = set((i.feature_name, i.matching_key, i.treatment) for i in impressions)
         assert as_tup_set == set(to_validate)
 
+    def _validate_last_events(self, client, *to_validate):
+        """Validate the last N impressions are present disregarding the order."""
+        event_storage = client._factory._get_storage('events')
+        events = event_storage.pop_many(len(to_validate))
+        as_tup_set = set((i.key, i.traffic_type_name, i.event_type_id, i.value, str(i.properties)) for i in events)
+        assert as_tup_set == set(to_validate)
+
     def test_get_treatment(self):
         """Test client.get_treatment()."""
         client = self.factory.client()
@@ -481,6 +558,17 @@ class InMemoryOptimizedIntegrationTests(object):
         assert len(manager.split_names()) == 7
         assert len(manager.splits()) == 7
 
+    def test_track(self):
+        """Test client.track()."""
+        client = self.factory.client()
+        assert(client.track('user1', 'user', 'conversion', 1, {"prop1": "value1"}))
+        assert(not client.track(None, 'user', 'conversion'))
+        assert(not client.track('user1', None, 'conversion'))
+        assert(not client.track('user1', 'user', None))
+        self._validate_last_events(
+            client,
+            ('user1', 'user', 'conversion', 1, "{'prop1': 'value1'}")
+        )
 
 class RedisIntegrationTests(object):
     """Redis storage-based integration tests."""
@@ -511,16 +599,41 @@ class RedisIntegrationTests(object):
         redis_client.sadd(segment_storage._get_key(data['name']), *data['added'])
         redis_client.set(segment_storage._get_till_key(data['name']), data['till'])
 
+        telemetry_redis_storage = RedisTelemetryStorage(redis_client, metadata)
+        telemetry_producer = TelemetryStorageProducer(telemetry_redis_storage)
+        telemetry_consumer = TelemetryStorageConsumer(telemetry_redis_storage)
+        telemetry_runtime_producer = telemetry_producer.get_telemetry_runtime_producer()
+
         storages = {
             'splits': split_storage,
             'segments': segment_storage,
             'impressions': RedisImpressionsStorage(redis_client, metadata),
             'events': RedisEventsStorage(redis_client, metadata),
         }
-        impmanager = ImpressionsManager(None, StrategyDebugMode())
-        recorder = PipelinedRecorder(redis_client.pipeline, impmanager,
-                                     storages['events'], storages['impressions'])
-        self.factory = SplitFactory('some_api_key', storages, True, recorder)  # pylint:disable=attribute-defined-outside-init
+        impmanager = ImpressionsManager(StrategyDebugMode(), telemetry_runtime_producer) # no listener
+        recorder = PipelinedRecorder(redis_client.pipeline, impmanager, storages['events'],
+                                    storages['impressions'], telemetry_redis_storage)
+        self.factory = SplitFactory('some_api_key',
+                                    storages,
+                                    True,
+                                    recorder,
+                                    telemetry_producer=telemetry_producer,
+                                    telemetry_init_producer=telemetry_producer.get_telemetry_init_producer(),
+                                    )  # pylint:disable=attribute-defined-outside-init
+
+    def _validate_last_events(self, client, *to_validate):
+        """Validate the last N impressions are present disregarding the order."""
+        event_storage = client._factory._get_storage('events')
+        redis_client = event_storage._redis
+        events_raw = [
+            json.loads(redis_client.lpop(event_storage._EVENTS_KEY_TEMPLATE))
+            for _ in to_validate
+        ]
+        as_tup_set = set(
+            (i['e']['key'], i['e']['trafficTypeName'], i['e']['eventTypeId'], i['e']['value'], str(i['e']['properties']))
+            for i in events_raw
+        )
+        assert as_tup_set == set(to_validate)
 
     def _validate_last_impressions(self, client, *to_validate):
         """Validate the last N impressions are present disregarding the order."""
@@ -706,9 +819,24 @@ class RedisIntegrationTests(object):
             ('sample_feature', 'invalidKey', 'off'),
         )
 
+    def test_track(self):
+        """Test client.track()."""
+        client = self.factory.client()
+        assert(client.track('user1', 'user', 'conversion', 1, {"prop1": "value1"}))
+        assert(not client.track(None, 'user', 'conversion'))
+        assert(not client.track('user1', None, 'conversion'))
+        assert(not client.track('user1', 'user', None))
+        self._validate_last_events(
+            client,
+            ('user1', 'user', 'conversion', 1, "{'prop1': 'value1'}")
+        )
+
     def test_manager_methods(self):
         """Test manager.split/splits."""
-        manager = self.factory.manager()
+        try:
+            manager = self.factory.manager()
+        except:
+            pass
         result = manager.split('all_feature')
         assert result.name == 'all_feature'
         assert result.traffic_type is None
@@ -788,17 +916,28 @@ class RedisWithCacheIntegrationTests(RedisIntegrationTests):
         redis_client.sadd(segment_storage._get_key(data['name']), *data['added'])
         redis_client.set(segment_storage._get_till_key(data['name']), data['till'])
 
+        telemetry_redis_storage = RedisTelemetryStorage(redis_client, metadata)
+        telemetry_producer = TelemetryStorageProducer(telemetry_redis_storage)
+        telemetry_consumer = TelemetryStorageConsumer(telemetry_redis_storage)
+        telemetry_runtime_producer = telemetry_producer.get_telemetry_runtime_producer()
+        telemetry_runtime_producer = telemetry_producer.get_telemetry_runtime_producer()
+
         storages = {
             'splits': split_storage,
             'segments': segment_storage,
             'impressions': RedisImpressionsStorage(redis_client, metadata),
             'events': RedisEventsStorage(redis_client, metadata),
         }
-        impmanager = ImpressionsManager(None, StrategyDebugMode())
+        impmanager = ImpressionsManager(StrategyDebugMode(), telemetry_runtime_producer) # no listener
         recorder = PipelinedRecorder(redis_client.pipeline, impmanager,
-                                     storages['events'], storages['impressions'])
-        self.factory = SplitFactory('some_api_key', storages, True, recorder)  # pylint:disable=attribute-defined-outside-init
-
+                                     storages['events'], storages['impressions'], telemetry_redis_storage)
+        self.factory = SplitFactory('some_api_key',
+                                    storages,
+                                    True,
+                                    recorder,
+                                    telemetry_producer=telemetry_producer,
+                                    telemetry_init_producer=telemetry_producer.get_telemetry_init_producer(),
+                                    )  # pylint:disable=attribute-defined-outside-init
 
 class LocalhostIntegrationTests(object):  # pylint: disable=too-few-public-methods
     """Client & Manager integration tests."""
